@@ -1,11 +1,14 @@
 import json
 import logging
 import threading
+from concurrent.futures import ThreadPoolExecutor, wait
 
+from .evaluation_details import EvaluationReason
 from .statsig_error_boundary import _StatsigErrorBoundary
 from .statsig_errors import StatsigValueError, StatsigNameError
 from .statsig_network import _StatsigNetwork
 from .statsig_options import StatsigOptions
+from .thread_util import spawn_background_thread
 
 RULESETS_SYNC_INTERVAL = 10
 IDLISTS_SYNC_INTERVAL = 60
@@ -24,31 +27,39 @@ def _is_specs_json_valid(specs_json):
 class _SpecStore:
     def __init__(self, network: _StatsigNetwork, options: StatsigOptions, statsig_metadata: dict,
                  error_boundary: _StatsigErrorBoundary, shutdown_event: threading.Event):
+        self.last_update_time = 0
+        self.initial_update_time = 0
+        self.init_reason = EvaluationReason.uninitialized
+
         self._network = network
         self._options = options
         self._statsig_metadata = statsig_metadata
         self._error_boundary = error_boundary
         self._shutdown_event = shutdown_event
+        self._executor = ThreadPoolExecutor(options.idlist_threadpool_size)
 
         self._configs = dict()
         self._gates = dict()
         self._layers = dict()
         self._experiment_to_layer = dict()
-        self._last_update_time = 0
 
         self._id_lists = dict()
 
         if not options.local_mode:
             self._initialize_specs()
+            self.initial_update_time = -1 if self.last_update_time == 0 else self.last_update_time
             self._download_id_lists()
 
-            self._background_download_configs = self._spawn_background_sync_thread(
-                (self._download_config_specs, options.rulesets_sync_interval or RULESETS_SYNC_INTERVAL))
-            self._background_download_idlists = self._spawn_background_sync_thread(
-                (self._download_id_lists, options.idlists_sync_interval or IDLISTS_SYNC_INTERVAL))
+            self._background_download_configs = spawn_background_thread(self._sync, (
+                self._download_config_specs, options.rulesets_sync_interval or RULESETS_SYNC_INTERVAL
+            ), error_boundary)
+
+            self._background_download_idlists = spawn_background_thread(self._sync, (
+                self._download_id_lists, options.idlists_sync_interval or IDLISTS_SYNC_INTERVAL
+            ), error_boundary)
 
     def is_ready_for_checks(self):
-        return self._last_update_time != 0
+        return self.last_update_time != 0
 
     def shutdown(self):
         if self._options.local_mode:
@@ -91,7 +102,7 @@ class _SpecStore:
                     "data_store gets priority over bootstrap_values. bootstrap_values will be ignored")
 
             self._load_config_specs_from_storage_adapter()
-            if self._last_update_time is 0:
+            if self.last_update_time == 0:
                 self._download_config_specs()
 
         elif self._options.bootstrap_values is not None:
@@ -100,9 +111,9 @@ class _SpecStore:
         else:
             self._download_config_specs()
 
-    def _process_specs(self, specs_json):
+    def _process_specs(self, specs_json) -> bool:
         if not _is_specs_json_valid(specs_json):
-            return
+            return False
 
         def get_parsed_specs(key: str):
             parsed = dict()
@@ -127,11 +138,13 @@ class _SpecStore:
         self._configs = new_configs
         self._layers = new_layers
         self._experiment_to_layer = new_experiment_to_layer
-        self._last_update_time = specs_json.get("time", 0)
+        self.last_update_time = specs_json.get("time", 0)
 
         if callable(self._options.rules_updated_callback):
             self._options.rules_updated_callback(json.dumps(specs_json))
 
+        return True
+
     def _bootstrap_config_specs(self):
         if self._options.bootstrap_values is None:
             return
@@ -141,7 +154,11 @@ class _SpecStore:
             if specs is None:
                 return
 
-            self._process_specs(specs)
+            if not _is_specs_json_valid(specs):
+                return
+
+            if self._process_specs(specs):
+                self.init_reason = EvaluationReason.bootstrap
         except ValueError:
             # JSON decoding failed, just let background thread update rulesets
             logging.getLogger('statsig.sdk').exception(
@@ -151,14 +168,15 @@ class _SpecStore:
     def _download_config_specs(self):
         specs = self._network.post_request("download_config_specs", {
             "statsigMetadata": self._statsig_metadata,
-            "sinceTime": self._last_update_time,
+            "sinceTime": self.last_update_time,
         })
 
         if not _is_specs_json_valid(specs):
             return
 
-        self._process_specs(specs)
-        self._save_to_storage_adapter(specs)
+        if self._process_specs(specs):
+            self._save_to_storage_adapter(specs)
+            self.init_reason = EvaluationReason.network
 
     def _save_to_storage_adapter(self, specs):
         if not _is_specs_json_valid(specs):
@@ -167,7 +185,7 @@ class _SpecStore:
         if self._options.data_store is None:
             return
 
-        if self._last_update_time is 0:
+        if self.last_update_time == 0:
             return
 
         self._options.data_store.set(STORAGE_ADAPTER_KEY, json.dumps(specs))
@@ -186,10 +204,11 @@ class _SpecStore:
             return
 
         adapter_time = cache.get("time", None)
-        if not isinstance(adapter_time, int) or adapter_time < self._last_update_time:
+        if not isinstance(adapter_time, int) or adapter_time < self.last_update_time:
             return
 
-        self._process_specs(cache)
+        if self._process_specs(cache):
+            self.init_reason = EvaluationReason.data_adapter
 
     def _download_id_lists(self):
         try:
@@ -200,7 +219,7 @@ class _SpecStore:
                 return
 
             local_id_lists = self._id_lists
-            thread_pool = []
+            workers = []
 
             for list_name in server_id_lists:
                 server_list = server_id_lists.get(list_name, dict())
@@ -231,15 +250,14 @@ class _SpecStore:
                 #  only download additional ids if sizes don't match
                 if size <= read_bytes or url == "":
                     continue
-                thread = threading.Thread(
-                    target=self._download_single_id_list,
-                    args=(url, list_name, local_list, local_id_lists, read_bytes,))
-                thread.daemon = True
-                thread_pool.append(thread)
-                thread.start()
 
-            for thread in thread_pool:
-                thread.join()
+                future = self._executor.submit(
+                    self._download_single_id_list,
+                    url, list_name, local_list, local_id_lists, read_bytes
+                )
+                workers.append(future)
+
+            wait(workers, self._options.idlists_sync_interval)
 
             deleted_lists = []
             for list_name in local_id_lists:
@@ -283,12 +301,6 @@ class _SpecStore:
         except Exception as e:
             self._error_boundary.log_exception(e)
 
-    def _spawn_background_sync_thread(self, args=()):
-        thread = threading.Thread(target=self._sync, args=args)
-        thread.daemon = True
-        thread.start()
-        return thread
-
     def _sync(self, sync_func, interval):
         while True:
             try:
