@@ -1,33 +1,39 @@
+import collections
 import threading
-import queue
 
+from .evaluation_details import EvaluationDetails
 from .evaluator import _ConfigEvaluation
 from .statsig_event import StatsigEvent
 from .layer import Layer
+from .thread_util import spawn_background_thread
 
 _CONFIG_EXPOSURE_EVENT = "statsig::config_exposure"
 _LAYER_EXPOSURE_EVENT = "statsig::layer_exposure"
 _GATE_EXPOSURE_EVENT = "statsig::gate_exposure"
 
+
+def _safe_add_evaluation_to_event(evaluation_details: EvaluationDetails, event: StatsigEvent):
+    if evaluation_details is None:
+        return
+
+    event.metadata["reason"] = evaluation_details.reason
+    event.metadata["configSyncTime"] = evaluation_details.config_sync_time
+    event.metadata["initTime"] = evaluation_details.init_time
+    event.metadata["serverTime"] = evaluation_details.server_time
+
+
 class _StatsigLogger:
     def __init__(self, net, shutdown_event, statsig_metadata, error_boundary, local_mode, event_queue_size):
         self._events = list()
-        self._retry_logs = queue.Queue(maxsize=10)
+        self._retry_logs = collections.deque(maxlen=10)
         self._net = net
         self._statsig_metadata = statsig_metadata
         self._local_mode = local_mode
         self._event_queue_size = event_queue_size
         self._error_boundary = error_boundary
 
-        self._background_flush = threading.Thread(
-            target=self._periodic_flush, args=(shutdown_event,))
-        self._background_flush.daemon = True
-        self._background_flush.start()
-
-        self._background_retry = threading.Thread(
-            target=self._periodic_retry, args=(shutdown_event,))
-        self._background_retry.daemon = True
-        self._background_retry.start()
+        self._background_flush = spawn_background_thread(self._periodic_flush, (shutdown_event,), error_boundary)
+        self._background_retry = spawn_background_thread(self._periodic_retry, (shutdown_event,), error_boundary)
 
     def log(self, event):
         if self._local_mode:
@@ -36,7 +42,7 @@ class _StatsigLogger:
         if len(self._events) >= self._event_queue_size or not self._background_flush.is_alive:
             self._flush()
 
-    def log_gate_exposure(self, user, gate, value, rule_id, secondary_exposures):
+    def log_gate_exposure(self, user, gate, value, rule_id, secondary_exposures, evaluation_details: EvaluationDetails):
         event = StatsigEvent(user, _GATE_EXPOSURE_EVENT)
         event.metadata = {
             "gate": gate,
@@ -46,9 +52,11 @@ class _StatsigLogger:
         if secondary_exposures is None:
             secondary_exposures = []
         event._secondary_exposures = secondary_exposures
+
+        _safe_add_evaluation_to_event(evaluation_details, event)
         self.log(event)
 
-    def log_config_exposure(self, user, config, rule_id, secondary_exposures):
+    def log_config_exposure(self, user, config, rule_id, secondary_exposures, evaluation_details: EvaluationDetails):
         event = StatsigEvent(user, _CONFIG_EXPOSURE_EVENT)
         event.metadata = {
             "config": config,
@@ -58,6 +66,8 @@ class _StatsigLogger:
         if secondary_exposures is None:
             secondary_exposures = []
         event._secondary_exposures = secondary_exposures
+
+        _safe_add_evaluation_to_event(evaluation_details, event)
         self.log(event)
 
     def log_layer_exposure(self, user, layer: Layer, parameter_name: str, config_evaluation: _ConfigEvaluation):
@@ -79,6 +89,8 @@ class _StatsigLogger:
         }
 
         event._secondary_exposures = [] if exposures is None else exposures
+
+        _safe_add_evaluation_to_event(config_evaluation.evaluation_details, event)
         self.log(event)
 
     def _flush(self):
@@ -91,7 +103,7 @@ class _StatsigLogger:
             "statsigMetadata": self._statsig_metadata,
         })
         if res is not None:
-            self._retry_logs.put(res, False)
+            self._retry_logs.append(res)
 
     def shutdown(self):
         self._flush()
@@ -111,9 +123,14 @@ class _StatsigLogger:
         while True:
             if shutdown_event.wait(60):
                 break
-            for i in range(self._retry_logs.qsize()):
-                payload = self._retry_logs.get()
+
+            length = len(self._retry_logs)
+            for i in range(length):
+                try:
+                    payload = self._retry_logs.pop()
+                except IndexError:
+                    break
+
                 res = self._net.retryable_request("log_event", payload)
                 if res is not None:
-                    self._retry_logs.put(res)
-                self._retry_logs.task_done()
+                    self._retry_logs.append(res)
